/* -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include "Ringbuffer.h"

#include <string.h>
#include <iostream>

using namespace fuppes;


Ringbuffer::Ringbuffer(size_t size)
{
  m_size = size;
  m_buffer = new unsigned char[size];

  m_read = 0;
  m_write = 0;
  m_empty = true;
}

Ringbuffer::~Ringbuffer()
{
  delete[] m_buffer;
}

void Ringbuffer::clear()
{
  m_read = 0;
  m_write = 0;
  m_empty = true;
}

size_t Ringbuffer::spaceLeft() const
{
  // empty or full
  if(m_read == m_write)
    return (m_empty ? m_size : 0);


  if(m_read <= m_write) {
    return (m_size - m_write) + m_read;
  }
  else if(m_read > m_write) {
    return (m_read - m_write);
  }

  return 0;
}

size_t Ringbuffer::spaceFilled() const
{
  return (m_size - spaceLeft());
}

size_t Ringbuffer::read(unsigned char* data, size_t size)
{
  if(spaceFilled() < size) {
    std::cout << "not enough bytes to read: " << spaceFilled() << std::endl;
    return 0;
  }


  size_t rest = m_size - m_read;
  if(rest > size)
    rest = size;

  if(rest > 0) {
    //std::cout << "buff1 read " << rest << std::endl;
    memcpy(data, &m_buffer[m_read], rest);
    m_read += rest;
  }

  if(m_read == m_size)
    m_read = 0;


  if(rest < size) {

    if(m_read != 0)
      std::cout << "FATAL READ" << std:: endl;

    size_t offset = rest;
    rest = size - rest;

    //std::cout << "buff2 read " << rest << std::endl;
    memcpy(&data[offset], &m_buffer[m_read], rest);
    m_read += rest;
  }


  m_empty = (m_write == m_read);
  return size;
}

size_t Ringbuffer::write(const unsigned char* data, size_t size)
{
  // not enough space left
  if(spaceLeft() < size) {
    std::cout << "not enough space left: " << spaceLeft() << std::endl;
    return 0;
  }



  size_t rest = m_size - m_write;
  if(rest > size)
    rest = size;

  if(rest > 0) {
    //std::cout << "buff1 write " << rest << std::endl;
    memcpy(&m_buffer[m_write], data, rest);
    m_write += rest;
  }


  if(m_write == m_size)
    m_write = 0;


  if(rest < size) {


    if(m_write != 0)
      std::cout << "FATAL WRITE" << std::endl;

    size_t offset = rest;
    rest = size - rest;

    //std::cout << "buff2 write " << rest << std::endl;
    memcpy(&m_buffer[m_write], &data[offset], rest);
    m_write = rest;
  }

  m_empty = !(m_write == m_read);
  return size;
}
