/* -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include "Log.h"

#include "Common/Thread.h"
#include "Common/Exception.h"

#include <iostream>
#include <ios>
#include <stdio.h>
#include <assert.h>

using namespace std;
using namespace fuppes;

std::string Log::senderToString(Log::Sender sender) // static
{
	switch(sender) {
    case Log::unknown:
      return "unknown";
    case Log::http:
      return "http";
    case Log::soap:
      return "soap";
    case Log::gena:
      return "gena";
    case Log::ssdp:
      return "ssdp";
    case Log::fam:
      return "fam";

    case Log::contentdir:
      return "contentdir";
    case Log::contentdb:
      return "contentdb";
    case Log::sql:
      return "sql";

    case Log::plugin:
      return "plugin";

    case Log::config:
      return "config";

    case Log::hotplug:
      return "hotplug";
      
    case Log::all:
      return "all";
		default:
			return "unknown";
	};	
}

Log::Sender Log::stringToSender(std::string sender) // static
{
  if(sender == "unknown")
    return Log::unknown;
  else if(sender == "http")
    return Log::http;
  else if(sender == "soap")
    return Log::soap;
  else if(sender == "gena")
    return Log::gena;
  else if(sender == "ssdp")
    return Log::ssdp;
  else if(sender == "fam")
    return Log::fam;
  
  else if(sender == "contentdir")
    return Log::contentdir;
  else if(sender == "contentdb")
    return Log::contentdb;
  else if(sender == "sql")
    return Log::sql;
  
  else if(sender == "plugin")
    return Log::plugin;

  else if(sender == "config")
    return Log::config;

  else if(sender == "hotplug")
    return Log::hotplug;

  else if(sender == "all")
    return Log::all;
  else
    return Log::unknown;
}



Log* Log::m_instance = NULL;

/*
void Log::init() // static
{
  if(m_instance != NULL)
    return;
  
  m_instance = new Log();
  m_instance->m_logSenders |= Log::hotplug;
}

void Log::uninit() // static
{
  if(m_instance == NULL)
    return;
  
  delete m_instance;
  m_instance = NULL;
}
*/

void Log::log_(Log::Sender sender, Log::Level level, const std::string fileName, int lineNo, const char* format, ...) // static
{
  bool active = isActiveSender(sender);

  if(!active) {
    //cout << "Log sender: " << Log::senderToString(sender) << " not active" << endl;
    return;
  }
  
	va_list args;
  va_start(args, format);
  Log::log_(sender, level, fileName, lineNo, format, args);
	va_end(args);
}


void Log::log_(Log::Sender sender, Log::Level level, const std::string fileName, int lineNo, const char* format, va_list args) // static
{
  bool active = isActiveSender(sender);

  if(!active) {
    //cout << "Log sender: " << Log::senderToString(sender) << " not active" << endl;
    return;
  }
  
	char buffer[8192 * 10];	
 	string out = "[" + Log::senderToString(sender) + "] ";
  vsnprintf(buffer, sizeof(buffer) - 1, format, args);
	
	cout << out << buffer << endl;
}



void Log::log_(Log::Sender sender, Log::Level level, const std::string fileName, int lineNo, const std::string msg) // static
{
  bool active = isActiveSender(sender);

  if(!active) {
    //cout << "Log sender: " << Log::senderToString(sender) << " not active" << endl;
    return;
  }

	string out = "[" + Log::senderToString(sender) + "] " + msg;
	//CSharedLog::Log(0, fileName, lineNo, out);
  cout << out << endl;
}



void Log::error_(Log::Sender sender, Log::Level level, const std::string fileName, int lineNo, const char* format, ...) // static
{
	va_list args;
  va_start(args, format);

  char buffer[8192 * 10];	
 	string out = "[" + Log::senderToString(sender) + "] ";
  vsnprintf(buffer, sizeof(buffer) - 1, format, args);
	va_end(args);
	
	cout << out << buffer << endl;
}

LogMgr* LogMgr::m_instance = NULL;

void LogMgr::init() // static
{
  assert(m_instance == NULL);
  m_instance = new LogMgr();
}

void LogMgr::uninit() // static
{
  assert(m_instance != NULL);
  delete m_instance;
  m_instance = NULL;
}

LogMgr* LogMgr::instance() // static
{
  assert(m_instance != NULL);
  return m_instance;  
}

LogMgr::LogMgr()
:m_ringbuffer(1024 * 128)
{
  m_active = false;
}

void LogMgr::append(const std::string msg) 
{  
  if(m_active) {
    m_mutex.lock();
    m_ringbuffer.write((const unsigned char*)msg.c_str(), msg.length());
    m_mutex.unlock();
  }

  //cout << msg << endl;
}

void LogMgr::read(std::string &out)
{
  m_mutex.lock();
  size_t size = m_ringbuffer.spaceFilled();
  char* data = new char[size];
  m_ringbuffer.read((unsigned char*)data, size);
  out = data;
  delete[] data;

  m_mutex.unlock();
}
size_t LogMgr::size() const
{
  return m_ringbuffer.spaceFilled();
}

void Log::init() // static
{
  LogMgr::init();
  
  m_instance = new Log();
  m_instance->m_logSenders |= (Log::hotplug | Log::contentdb);
}

void Log::uninit() // static
{
  delete m_instance;
  m_instance = NULL;

  LogMgr::uninit();
}

Log::Log(Log::Sender sender, Log::Level level, const std::string filename, int lineNo)
{
  m_sender = sender;
  m_level = level;
  m_filename = filename;
  m_lineNo = lineNo;

  Thread* thread = ThreadPool::threadById(Thread::currentThreadId());

  if(thread != NULL) {
    m_stream << "[ Thread: " << thread->name() << " " << std::hex << thread->threadId() << " ] " << std::dec;
  }
  else {
    m_stream << "[ NOTHREAD ] ";
  }
  
  m_stream << filename << " " << lineNo << " :: ";
}

Log::~Log()
{
  if(m_instance != NULL)
    LogMgr::instance()->append(m_stream.str());
}



Log &Log::operator<<(fuppes::Exception e)
{ 
  m_stream << "EXCEPTION: " << e.what(); return space(); 
}
